clear
clc

mpc = loadcase(case4gs);

% Define named indices into bus, gen, branch matrices:
defineNamedIndices;

% Copy:
[baseMVA, bus, gen, branch] = ...
    deal(mpc.baseMVA, mpc.bus, mpc.gen, mpc.branch);

% Create reference:
mpopt = mpoption('model','DC');
[r1, s1] = runpf(mpc, mpopt);

Va0 = r1.bus(:,VA) * pi/180;

% Build B-matrix:
[Bbus,~,~,~] = makeBdc(baseMVA, bus, branch);

% Builds index lists for each type of bus (REF, PV, PQ):
[ref,pv,pq] = bustypes(bus, gen);

% Factorization:
Bt = Bbus([pv;pq],[pv;pq]);
[L, U, P, Q] = lu(Bt);

nb = length(bus(:,1));
nl = length(branch(:,1));

bf0 = branch(:,F_BUS);
bt0 = branch(:,T_BUS);

% Maximum  200% loading:
maxVa = zeros(nl,1);
for i = 1:nl
maxVa(i) = 2.0 * abs(Va0(bt0(i))-Va0(bf0(i)));
end

% Compensation:
success = ones(nl,2);
success(:,1) = 1:nl;
resVa = zeros(nb,nl);
Va = zeros(nl,nl);
for i = 1:nl

bf = branch(i,F_BUS);
bt = branch(i,T_BUS);

M = sparse([bf bt],1,[1 -1],nb,1);
M = M([pv;pq]); 

invBbusM = Q * (U \ (L \ (P * M )));
z = M' * invBbusM;
dy = Bbus(bf,bt);
c = 1 / (1/dy + z);

deltaVa = invBbusM * c * M' * Va0([pv;pq]); 

VaComp = Va0([pv;pq]) - deltaVa;

resVa([pv;pq],i) = VaComp;

% Check if angle difference is violated:

for j = 1:nl
    Va(j,i) = abs(resVa(bt0(j),i)-resVa(bf0(j),i));
    if  Va(j,i) > maxVa(j) && Va(j,i) > 0.02 && i~=j
        success(i,2) = 0;
    end
end

end
% Display success matrix:
success 
resVa
Va0
Va









