clear
clc

mpc = loadcase(case4gs);

% Define named indices into bus, gen, branch matrices:
defineNamedIndices;

% Split up:
[baseMVA, bus, gen, branch] = ...
    deal(mpc.baseMVA, mpc.bus, mpc.gen, mpc.branch);

% Create reference:
mpopt = mpoption('model','DC');
[results1, success1] = runpf(mpc, mpopt);

Va0 = results1.bus(:,VA) * pi/180;

% Build B-matrix:
[Bbus, ~, ~, ~] = makeBdc(baseMVA, bus, branch);

% Builds index lists for each type of bus (REF, PV, PQ):
[ref, pv, pq] = bustypes(bus, gen);

% Factorization:
Bt = Bbus([pv; pq],[pv; pq]);
[L, U, P, Q] = lu(Bt);

nb = length(bus(:,1));
resVa = zeros(nb,1);

%% Compensation:
% Branch index of line:
b1 = 1;

bf1 = branch(b1,F_BUS);
bt1 = branch(b1,T_BUS);

M = sparse([bf1 bt1],1,[1 -1],nb,1);
M = M([pv;pq]); 

invBbusM = Q * (U \ (L \ (P * M )));
z = M' * invBbusM;
dy = Bbus(bf1,bt1);
c = 1 / ((1/dy) + z);

deltaVa = invBbusM * c * M' * Va0([pv;pq]); 

VaComp = Va0([pv;pq]) - deltaVa;
VaComp = VaComp * 180/pi;

resVa([pv;pq]) = VaComp;

% New flows:
mpc.branch(b1,BR_STATUS) = 0;
[results2, success2] = runpf(mpc, mpopt);

comparsion = [results2.bus(:,VA) resVa]


