clc
clear
format shortg

%% Primal-version (BX version starting with P-iteration)
%------------v------del----Pg-----Qg-----Pd-----Qd----type-- 
bus = [1   1.01    0.00   0.00   0.00   0.00   0.00    1 ;
       2   1.02    0.00   1.00   0.00   0.00   0.00    2 ;
       3   1.02    0.00   0.80   0.00   0.00   0.00    2 ;
       4   1.00    0.00   0.00   0.80   0.00   0.00    3 ;
       5   1.00    0.00   0.00   0.00   0.40   0.00    3 ;
       6   1.00    0.00   0.00   0.00   0.50   0.00    3 ;
       7   1.00    0.00   0.00   0.00   0.30   0.00    3 ;
       8   1.00    0.00   0.00   0.00   0.40   0.00    3 ;
       9   1.00    0.00   0.00   0.00   0.40   0.50    3];
      
%-----from--to-----r--------x---     
line = [1   2   0.0546   0.2112 ;
        2   3   0.0218   0.0845 ;
    	3   4   0.0328   0.1267 ;
    	4   5   0.0546   0.2112 ;
    	5   6   0.0218   0.0845 ;
    	6   7   0.0328   0.1267 ;
        7   8   0.0546   0.2112 ;
    	8   9   0.0218   0.0845 ;
    	9   1   0.0328   0.1267 ;
        1   4   0.0546   0.2112 ;
        5   7   0.0218   0.0845 ;
        8   1   0.0328   0.1267 ];
    
buses = max(bus(:,1));
[temp1,~] = size(line);
lines = temp1;
    
% Update line matrix with chosen x/r-ratio:
R = 1.0;
X = 0.15;
line(:,3) = line(:,3) .* R;
line(:,4) = line(:,4) .* X;
 
% Prepare to create ybus-matrix & simplified ybus-matrix using 1/x:
ytemp = zeros(buses,buses);
xtemp = zeros(buses,buses);
for x = 1:lines                   
    i = line(x,1); % From
    j = line(x,2); % To   
    ytemp(i,j) = 1 / (line(x,3) + 1i*line(x,4)); % Admittance value
    ytemp(j,i) = ytemp(i,j); % Symmetric
    xtemp(i,j) = 1/(1i*line(x,4));
    xtemp(j,i) = xtemp(i,j);
end

% Build admittance matrix:
ybus = zeros(buses,buses);
xbus = zeros(buses,buses);
for i = 1:buses
    for j = 1:buses
        if i==j
            ybus(i,j) = sum(ytemp(i,:)); 
            xbus(i,j) = sum(xtemp(i,:)); 
        else
            ybus(i,j) = -1*ytemp(i,j);
            xbus(i,j) = -1*xtemp(i,j);
        end
    end
end

% Build b' matrtix:
b = -imag(ybus);
b1 = zeros(buses-1,buses-1);
for i = 1:buses-1
    for j = 1:buses-1
        b1(i,j) = b(i+1,j+1);
    end
end

% Build b" matrix:
b = -imag(xbus);
b2 = zeros(buses-3,buses-3);
for i = 1:(buses-3)
    for j = 1:(buses-3)
        b2(i,j) = b(i+3,j+3);
    end
end

V = bus(:,2);
del = bus(:,3);
Pg = bus(:,4);
Qg = bus(:,5);
Pd = bus(:,6);
Qd = bus(:,7);
Pspec = Pg-Pd;
Qspec = Qg-Qd;
G = real(ybus); % Conductance
B = imag(ybus); % Susceptance

iter = 0;
tolerance = 0.001;
condition = 1;
while condition == 1;  
    P = zeros(buses,1);
    Q = zeros(buses,1);
    iter = iter+1;

% Active power: 
for i=1:buses     
    for j=1:buses
        P(i) = P(i) + (V(i)*V(j)*(G(i,j)*cos(del(i)-del(j)) ...
               +B(i,j)*sin(del(i)-del(j))));
    end
end

% Delta P divided by V:
for i = 1:(buses-1)   
    delP(i,1) = Pspec(i+1)-P(i+1);
    delPbyV(i,1) = delP(i,1) ./ V(i+1,1);
end

% Correction vector, angle:  
    deldel = b1 \ delPbyV;

% Update voltage angles:
for i = 1:(buses-1)
    del(i+1,1) = del(i+1,1) + deldel(i,1);
end

% Reactive power: 
for i=1:buses     
    for j=1:buses
        Q(i) = Q(i)+ (V(i)*V(j)*(G(i,j)*sin(del(i)-del(j)) ...
               -B(i,j)*cos(del(i)-del(j))));
    end
end

% Delta Q divided by V:
c=0;
for i=1:buses
    if bus(i,8) == 3
        c = c+1;
        delQ(c,1) = Qspec(i) - Q(i);
        delQbyV(c,1) = delQ(c,1) ./ V(i,1);
    end
end

% Correction vector, voltage magnitudes:    
    delv = b2 \ delQbyV;

% Update voltages:
c=0;
for i = 1:buses
    if bus(i,8) == 3
        c = c+1;
        V(i,1) = V(i,1) + delv(c,1);
    end
end

iter 
P
Q
V
del

if max(abs(delP)) < tolerance && max(abs(delQ)) < tolerance
    condition = 0;   
elseif iter == 30
    condition = 0;
end

end
Ploss = sum(P)