%% ~* Test angle updating in the efficient bounding Method*~
clear
clc
format shortg 

%% *Choose paramters*
branch_out = 22; % Interruption of this branch
node = 1; % Find out how the voltage-angle of this node change.

%% Define named indices into bus, gen, branch matrices:
defineNamedIndices;

%% System parameters:
casedata = case24_ieee_rts; %; % case2383wp;    case4gs  
mpc = loadcase(casedata);
[baseMVA, bus, gen, branch] = ... 
    deal(mpc.baseMVA, mpc.bus, mpc.gen, mpc.branch);
[ref, pv, pq] = bustypes(bus, gen);

%% Referene power flow:
mpopt = mpoption('model','DC');
[r1,s1] = runpf(mpc, mpopt);
f0 = r1.branch(:,PF) ./ baseMVA; % f_pq^0

%% Add zero columns to branch for flows if needed:
if size(mpc.branch,2) < QT
mpc.branch = [mpc.branch zeros(size(mpc.branch, 1), ...
    QT-size(mpc.branch,2))];
end

%% Build B matrices and phase shift injections:
[B, Bf, Pbusinj, Pfinj] = makeBdc(baseMVA, bus, branch);

%% Set values:
nb = size(bus,1); % Number of buses.
stat = branch(:,BR_STATUS); %% Ones at in-service branches.
x = branch(:,BR_X); %% Branch reactances.
indices = find(stat == 0);
x(indices) = NaN; % Disconnected branches.

%% Node lists of the branches:
bf = branch(:,F_BUS);
bt = branch(:,T_BUS);
% LUPQ factorization of B:
[L, U, P, Q] = lu(B([pv;pq],[pv;pq]));

%% Calculate the two inverse X-columns needed for each contingency:
if bf(branch_out) ~= ref
one_bf = sparse(bf(branch_out),1,1,nb,1);
X_bf([pv;pq]) = Q * (U \(L \(P * one_bf([pv;pq]))));
end

if bt(branch_out) ~= ref
one_bt = sparse(bt(branch_out),1,1,nb,1);
X_bt([pv;pq]) = Q * (U \(L \(P * one_bt([pv;pq]))));
end

delta = zeros(1,1);
if node == ref
delta(1) = 0;
elseif bt(branch_out) == ref
delta(1) = (X_bf(node)*x(branch_out))/(x(branch_out)-X_bf(bf(branch_out)));
elseif bf(branch_out) == ref
delta(1) = -(X_bt(node)*x(branch_out))/ ...
    (x(branch_out)-X_bt(bt(branch_out)));
else
delta(1) = (X_bf(node)-X_bt(node))*x(branch_out)/ ...
    (x(branch_out)-(X_bf(bf(branch_out))+ ...
    X_bt(bt(branch_out))-2*X_bt(bf(branch_out))));
end

angle = delta * f0(branch_out);

% New flows:
mpc.branch(branch_out,BR_STATUS) = 0;
[r2,s2] = runpf(mpc, mpopt);

%% Output:
voltageAngle0 = r1.bus(:,VA)
voltageAngle = r2.bus(:,VA)
angleChange = angle * 180/pi
updatedAngle = voltageAngle0(node) + angleChange 
branch_out
node


%

